import pdfkit
import pandas as pd
import os
from dominate.tags import html, body, h1, div
from dominate.util import raw
from openpyxl.styles import PatternFill, Font
from openpyxl.styles.differential import DifferentialStyle
from openpyxl import load_workbook
from openpyxl.formatting.rule import Rule
import json
import time


class FormatConfig:
    def __init__(self, config_file):
        with open(config_file) as jf:
            self._formatting = json.load(jf)

    def load_body_format(self):
        return self._formatting["body"]

    def load_table_format(self):
        return self._formatting["table"]

    def load_rpm_info_format(self):
        return self._formatting['highlight']['rpm-info']

    def load_support_flag_format(self):
        return self._formatting['highlight']['support-flag']

    def load_running_format(self):
        return self._formatting['highlight']['running']


class HTMLTableFormatting:
    def __init__(self, formatting_config_file='/config/soliddriver-checks.conf'):
        self._formatting = FormatConfig(formatting_config_file)

    def get_style(self):
        table_formatting = self._formatting.load_table_format()
        styles = [
            dict(selector='table',
                 props=[('border', table_formatting['border'])]),
            dict(selector='th',
                 props=[('border', table_formatting['th']['border']),
                        ('font-size', table_formatting['th']['font-size']),
                        ('font-family', table_formatting['th']['font-family']),
                        ('background-color',
                        table_formatting['th']['background-color'])]),
            dict(selector='td',
                 props=[('border', table_formatting['th']['border']),
                        ('font-size', table_formatting['td']['font-size']),
                        ('font-family',
                        table_formatting['td']['font-family'])])
            ]

        return styles


class RPMsExporter:
    def __init__(self, logger,
                 formatting_config_file='/config/soliddriver-checks.conf'):
        self._logger = logger
        self._formatting = FormatConfig(formatting_config_file)

    def _supported_formatting(self, value):
        formatting = self._formatting.load_support_flag_format()
        bgcolor_missing = formatting['Missing']['background-color']
        bgcolor_yes = formatting['yes']['background-color']
        bgcolor_external = formatting['external']['background-color']
        if ': no' in value or ': Missing' in value:
            return 'background-color: ' + bgcolor_missing
        elif ': yes' in value:
            return 'background-color: ' + bgcolor_yes
        elif ': external' in value:
            return 'background-color: ' + bgcolor_external
        else:
            return 'background-color: white'

    def to_html(self, rpm_table, file):
        tableFormatter = HTMLTableFormatting()
        styles = tableFormatter.get_style()

        rpm_table['Driver Flag: supported'] = rpm_table['Driver Flag: supported'].str.replace('\n', '</br>')
        rpm_table['Symbols Check'] = rpm_table['Symbols Check'].str.replace('\n', '</br>')

        s = rpm_table.style.applymap(self._supported_formatting,
                                     subset=pd.IndexSlice[:, ['Driver Flag: supported']],
                                     ).hide_index()

        s = s.set_table_styles(styles)

        with open(file, 'w') as f:
            f.write(s.render())

    def to_excel(self, rpm_table, file):
        writer = pd.ExcelWriter(file, engine='openpyxl')
        if os.path.exists(file):
            writer = pd.ExcelWriter(file, engine='openpyxl', mode='a')

        sheet_name = 'Solid Driver Checks'
        rpm_table.to_excel(writer,
                           index=False,
                           sheet_name=sheet_name)
        writer.save()
        writer.close()

        workbook = load_workbook(filename=file)
        worksheet = workbook[sheet_name]
        records = len(rpm_table.index) + 1

        if records < 1:
            workbook.save(file)

        area = 'F2:F' + str(records)
        support_flag_format = self._formatting.load_support_flag_format()
        na_text = Font(color=support_flag_format['Missing']['font-color'])
        na_fill = PatternFill(bgColor=support_flag_format['Missing']['background-color'])
        na = DifferentialStyle(font=na_text, fill=na_fill)
        yes_text = Font(color=support_flag_format['yes']['font-color'])
        yes_fill = PatternFill(bgColor=support_flag_format['yes']['background-color'])
        yes = DifferentialStyle(font=yes_text, fill=yes_fill)
        external_text = Font(color=support_flag_format['external']['font-color'])
        external_fill = PatternFill(bgColor=support_flag_format['external']['background-color'])
        external = DifferentialStyle(font=external_text, fill=external_fill)

        support_flag_na_rule = Rule(type='containsText',
                                    operator='containsText',
                                    text='Missing', dxf=na)
        support_flag_external_rule = Rule(type='containsText',
                                          operator='containsText',
                                          text='external', dxf=external)
        support_flag_yes_rule = Rule(type='containsText',
                                     operator='containsText',
                                     text='yes', dxf=yes)

        worksheet.conditional_formatting.add(area,
                                             support_flag_na_rule)
        worksheet.conditional_formatting.add(area,
                                             support_flag_external_rule)
        worksheet.conditional_formatting.add(area,
                                             support_flag_yes_rule)

        workbook.save(file)

    def to_pdf(self, rpm_table, file):
        self.to_html(rpm_table, '.tmp.rpms.html')
        pdfkit.from_file('.tmp.rpms.html', file)
        os.remove('.tmp.rpms.html')

    def to_all(self, rpm_table, directory):
        excel_file = os.path.join(directory, 'check_result.xlsx')
        html_file = os.path.join(directory, 'check_result.html')
        pdf_file = os.path.join(directory, 'check_result.pdf')

        if not os.path.exists(directory):
            os.mkdir(directory)
        else:
            if os.path.exists(excel_file):
                os.remove(excel_file)
            if os.path.exists(html_file):
                os.remove(html_file)
            if os.path.exists(pdf_file):
                os.remove(pdf_file)

        self.to_excel(rpm_table, excel_file)
        self.to_html(rpm_table, html_file)
        self.to_pdf(rpm_table, pdf_file)


class DriversExporter:
    def __init__(self, logger,
                 formatting_config_file='/config/soliddriver-checks.conf'):
        self._logger = logger
        self._formatting = FormatConfig(formatting_config_file)

    def _supported_color(self, value):
        if value == 'yes':
            return '[green]' + value + '[/green]'
        elif value == 'external':
            return '[blue]' + value + '[/blue]'
        else:
            return '[red]' + value + '[/red]'

    def _running_color(self, value):
        if value == 'True':
            return '[green]True[/green]'
        else:
            return '[gray]False[gray]'

    def _rpm_info_color(self, value):
        if 'is not owned by any package' in value:
            return '[red]' + value + '[/red]'
        else:
            return value

    def _supported_html_format_handler(self, value):
        value = value.lstrip().rstrip()
        supported = self._formatting.load_support_flag_format()
        bgcolor_yes = supported['yes']['background-color']
        bgcolor_external = supported['external']['background-color']
        bgcolor_missing = supported['Missing']['background-color']
        if value == 'yes':
            return 'background-color:%s' % bgcolor_yes
        elif value == 'external':
            return 'background-color:%s' % bgcolor_external
        elif value == 'Missing' or value == 'no':
            return 'background-color:%s' % bgcolor_missing

        return ''

    def _running_html_format_handler(self, value):
        running_format = self._formatting.load_running_format()
        bgcolor_true = running_format['true']['background-color']
        bgcolor_false = running_format['false']['background-color']
        if value == 'True':
            return 'background-color:%s' % bgcolor_true

        return 'background-color:%s' % bgcolor_false

    def _rpm_info_html_format_handler(self, value):
        rpm_format = self._formatting.load_rpm_info_format()
        bgcolor_no_rpm = rpm_format['no-rpm']['background-color']
        if 'is not owned by any package' in value:
            return 'background-color:%s' % bgcolor_no_rpm

        return ''

    def to_html(self, driver_tables, file):
        html_table_formatter = HTMLTableFormatting()
        styles = html_table_formatter.get_style()
        body_format = self._formatting.load_body_format()
        context = html()
        with context:
            font_family = body_format["font-family"]
            body_style = 'font-family: %s;' % font_family
            with body(style=body_style):
                for label, driver_table in driver_tables.items():
                    s = driver_table.style.\
                        applymap(self._supported_html_format_handler,
                                 subset=pd.IndexSlice[:, ['Flag: supported']]).\
                        applymap(self._running_html_format_handler,
                                 subset=pd.IndexSlice[:, ['Running']]).\
                        applymap(self._rpm_info_html_format_handler,
                                 subset=pd.IndexSlice[:, ['RPM Information']]).hide_index().\
                        set_table_styles(styles)

                    h1('Solid Driver Checking Result: %s' % label)
                    div(raw(s.render()))

        with open(file, 'w') as f:
            f.write(context.render())

    def to_excel(self, driver_tables, file):
        for server in driver_tables:
            writer = pd.ExcelWriter(file, engine='openpyxl')
            if os.path.exists(file):
                writer = pd.ExcelWriter(file, engine='openpyxl', mode='a')

            driver_tables[server].to_excel(writer, index=False,
                                           sheet_name=server)
            writer.save()
            writer.close()

            workbook = load_workbook(filename=file)
            worksheet = workbook[server]

            records = len(driver_tables[server].index)

            if records < 1:
                workbook.save(file)
                continue

            records = str(len(driver_tables[server].index))

            support_flag_area = 'C2:C' + records
            running_area = 'E2:E' + records
            rpm_info_area = 'F2:F' + records

            support_flag_format = self._formatting.load_support_flag_format()
            na_text = Font(color=support_flag_format['Missing']['font-color'])
            na_fill = PatternFill(bgColor=support_flag_format['Missing']['background-color'])
            na = DifferentialStyle(font=na_text, fill=na_fill)
            yes_text = Font(color=support_flag_format['yes']['font-color'])
            yes_fill = PatternFill(bgColor=support_flag_format['yes']['background-color'])
            yes = DifferentialStyle(font=yes_text, fill=yes_fill)
            external_text = Font(color=support_flag_format['external']['font-color'])
            external_fill = PatternFill(bgColor=support_flag_format['external']['background-color'])
            external = DifferentialStyle(font=external_text, fill=external_fill)
            running_format = self._formatting.load_running_format()
            false_text = Font(color=running_format['false']['font-color'])
            false_fill = PatternFill(bgColor=running_format['false']['background-color'])
            false_format = DifferentialStyle(font=false_text, fill=false_fill)
            true_text = Font(color=running_format['true']['font-color'])
            true_fill = PatternFill(bgColor=running_format['true']['background-color'])
            true_format = DifferentialStyle(font=true_text, fill=true_fill)
            rpm_info_format = self._formatting.load_rpm_info_format()
            no_rpm_text = Font(color=rpm_info_format['no-rpm']['font-color'])
            no_rpm_fill = PatternFill(bgColor=rpm_info_format['no-rpm']['background-color'])
            no_rpm = DifferentialStyle(font=no_rpm_text, fill=no_rpm_fill)
            support_flag_na_rule = Rule(type='containsText',
                                        operator='containsText',
                                        text='Missing', dxf=na)
            support_flag_external_rule = Rule(type='containsText',
                                              operator='containsText',
                                              text='external', dxf=external)
            support_flag_yes_rule = Rule(type='containsText',
                                         operator='containsText',
                                         text='yes', dxf=yes)
            running_yes_rule = Rule(type='containsText',
                                    operator='containsText',
                                    text='True', dxf=true_format)
            running_no_rule = Rule(type='containsText',
                                   operator='containsText',
                                   text='False', dxf=false_format)
            no_rpm_rule = Rule(type='containsText', operator='containsText',
                               text='is not owned by any package', dxf=no_rpm)

            worksheet.conditional_formatting.add(support_flag_area,
                                                 support_flag_na_rule)
            worksheet.conditional_formatting.add(support_flag_area,
                                                 support_flag_external_rule)
            worksheet.conditional_formatting.add(support_flag_area,
                                                 support_flag_yes_rule)
            worksheet.conditional_formatting.add(running_area,
                                                 running_yes_rule)
            worksheet.conditional_formatting.add(running_area,
                                                 running_no_rule)
            worksheet.conditional_formatting.add(rpm_info_area,
                                                 no_rpm_rule)

            workbook.save(file)

    def to_pdf(self, driver_tables, file):
        self.to_html(driver_tables, '.tmp.rpms.html')
        pdfkit.from_file('.tmp.rpms.html', file)
        os.remove('.tmp.rpms.html')

    def to_all(self, driver_tables, directory):
        excel_file = os.path.join(directory, 'check_result.xlsx')
        html_file = os.path.join(directory, 'check_result.html')
        pdf_file = os.path.join(directory, 'check_result.pdf')

        if not os.path.exists(directory):
            os.mkdir(directory)
        else:
            if os.path.exists(excel_file):
                os.remove(excel_file)
            if os.path.exists(html_file):
                os.remove(html_file)
            if os.path.exists(pdf_file):
                os.remove(pdf_file)

        self.to_excel(driver_tables, excel_file)
        self.to_html(driver_tables, html_file)
        self.to_pdf(driver_tables, pdf_file)



from utils import data_reader
from rich.progress import Progress
from paramiko.ssh_exception import NoValidConnectionsError


def check_remote_servers(logger, servers):
    check_result = dict()
    for server in servers:
        if server['check'] == 'False':
            continue

        logger.info("Start to analysis server: %s", server['ip'])
        try:
            with Progress() as progress:
                reader = data_reader.DriverReader(logger, progress)
                drivers = reader.get_remote_drivers(ip=server['ip'],
                                                    user=server['user'],
                                                    password=server['password'],
                                                    ssh_port=server['ssh_port'],
                                                    query=server['query'])
                check_result[server['ip']] = drivers
        except NoValidConnectionsError:
            logger.error("Can not connect to server: %s", server['ip'])
        finally:
            pass

    logger.info("Analysis is completed!")

    return check_result



import subprocess
from pathlib import Path
import os
import shutil
import paramiko
import pandas as pd
import select
import re
from collections import namedtuple


def get_cmd_all_drivers_modinfo():
    return '/usr/sbin/modinfo $(find /lib/modules/ -regex ".*\.\(ko\|ko.xz\)$")'


def get_cmd_all_running_drivers_modinfo():
    return '/usr/sbin/modinfo $(cat /proc/modules | awk \'{print $1}\')'


def async_run_cmd(cmd, line_handler,
                  line_handler_arg, start, condition, sshClient=None):
    if sshClient is not None:
        channel = sshClient.get_transport().open_session()
        channel.exec_command(cmd)

        while not channel.exit_status_ready():
            r, __, __ = select.select([channel], [], [])
            if len(r) > 0:
                recv = channel.recv(1024)
                recv = str(recv, 'utf-8').splitlines()
                for line in recv:
                    line_handler(line_handler_arg, line, start, condition)
                    start += 1

        channel.close()
        # del stdin, stdout, stderr
    else:
        cmd_runner = subprocess.Popen(cmd,
                                      shell=True,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE)
        for line in cmd_runner.stdout:
            line = str(line, 'utf-8')
            line_handler(line_handler_arg, line, start, condition)
            start += 1


def run_cmd(cmd, sshClient=None, timeout=None):
    if sshClient is not None:
        __, stdout, __ = sshClient.exec_command(cmd, timeout=timeout)

        result = stdout.read()

        return result
    else:
        cmd_runner = subprocess.Popen(cmd,
                                      shell=True,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE)
        result, errs = cmd_runner.communicate()

        return result


class RPMReader:
    def __init__(self, logger):
        self._logger = logger
        self._columns = ['Name',
                         'Path',
                         'Vendor',
                         'Signature',
                         'Distribution',
                         'Driver Flag: supported',
                         'Symbols Check']

    def _driver_symbols_check(self, rpm_symbols, driver):
        symvers = run_cmd('/usr/sbin/modprobe --dump-modversions %s' % driver)

        result = []
        for line in symvers.splitlines():
            line = str(line, 'utf-8')
            chksum, sym = line.split()
            chksum = hex(int(chksum, base=16))

            req = rpm_symbols.get(sym, None)
            if req is None:
                result.append('Symbol %s not found in rpm requires' % sym)
                continue

            if req.checksum != chksum:
                result.append('Symbol checksum does not match, module depends on %s, rpm requires %s' % (chksum, req.checksum))
                continue

        return result

    def _get_rpm_symbols(self, rpm):
        KernelSym = namedtuple('KernelSym', 'kernel_flavor symbol checksum')
        symver_re = re.compile(r'ksym\((.*):(.*)\) = (.+)')
        raw_symbols = run_cmd('rpm -q --requires %s' % rpm)

        mod_reqs = {}
        for line in raw_symbols.splitlines():
            line = str(line, 'utf-8')
            result = symver_re.match(line)
            if result:
                flavor, sym, chksum = result.groups()
                chksum = hex(int(chksum, base=16))
                mod_reqs[sym] = KernelSym(kernel_flavor=flavor,
                                          symbol=sym, checksum=chksum)

        return mod_reqs

    def _get_driver_supported(self, driver):
        raw_info = run_cmd('/usr/sbin/modinfo %s' % driver)
        raw_info = str(raw_info, 'utf-8')
        info_list = raw_info.splitlines()
        for item in info_list:
            values = item.split(':')
            if len(values) < 2:
                continue

            if values[0].rstrip() == 'supported':
                return ":".join(values[1:]).lstrip()

        return "Missing"

    def _drivers_terminal_format(self, drivers):
        supported = ""
        symbols = ""
        for driver in drivers:
            supported += "%s : %s\n" % (driver, drivers[driver]['supported'])
            mismatched_symbols = drivers[driver]['mismatch-symbols']
            if len(mismatched_symbols) > 0:
                symbols += "%s :\n" % driver
                for sym in mismatched_symbols:
                    symbols += "  %s\n" % sym

        return supported, symbols

    def _driver_checks(self, rpm: str):
        mod_reqs = self._get_rpm_symbols(rpm)

        Path('tmp').mkdir(parents=True, exist_ok=True)
        os.chdir('tmp')

        command = 'rpm2cpio %s | cpio -idmv' % rpm
        rpm_unpack = subprocess.Popen(command,
                                      shell=True,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE)
        rpm_unpack.wait()

        rpm_dir = Path('.')
        files = tuple(rpm_dir.rglob('*.*'))
        drivers = [i for i in files if re.search(r'\.(ko|xz\.ko)$', str(i))]
        result = dict()

        if len(drivers) < 1:
            os.chdir('../')
            shutil.rmtree('tmp')

            return None

        for driver in drivers:
            item = dict()
            item['mismatch-symbols'] = self._driver_symbols_check(mod_reqs, driver)
            item['supported'] = self._get_driver_supported(driver)

            result[str(driver)] = item

        os.chdir('../')
        shutil.rmtree('tmp')

        return result

    def _format_rpm_info(self, rpm_files,
                         raw_output, row_handlers, query='all'):
        raw_output = str(raw_output, 'utf-8').split("Name        :")
        rpms = raw_output[1:]

        for i, rpm in enumerate(rpm_files):
            info = rpms[i].splitlines()
            name = info[0].lstrip()
            signature = ''
            distribution = ''
            vendor = ''
            for item in info:
                values = item.split(':')
                if len(values) < 2:
                    continue

                if values[0].rstrip() == "Signature":
                    signature = ":".join(values[1:])
                elif values[0].rstrip() == "Distribution":
                    distribution = ":".join(values[1:])
                elif values[0].rstrip() == "Vendor":
                    vendor = ":".join(values[1:])

            driver_checks = self._driver_checks(rpm)

            supported = ""
            symbols = ""
            if driver_checks is not None:
                supported, symbols = self._drivers_terminal_format(driver_checks)

            if not self._query_filter(supported, query):
                continue

            for handler in row_handlers:
                handler([name, rpm, vendor, signature,
                         distribution, supported, symbols])

    def _query_filter(self, supported, query):
        if query == 'all':
            return True
        elif query == 'suse' and ': yes' in supported:
            return True
        elif query == 'vendor' and ': external' in supported:
            return True
        elif query == 'unknow' and ': Missing' in supported:
            return True

        return False

    def _add_row(self, row):
        self._rpm_df = self._rpm_df.append(pd.Series(row, index=self._columns),
                                           ignore_index=True)

    def get_rpms_info(self, path, row_handlers=None, query="all"):
        rpm_files = run_cmd('find %s -name "*.rpm"' % path)
        rpm_files = str(rpm_files, 'utf-8').splitlines()
        rpm_infos = run_cmd('rpm -qpi --nosignature $(find %s -name "*.rpm")' % path)

        if row_handlers is None:
            row_handlers = []

        self._rpm_df = pd.DataFrame(columns=self._columns)
        row_handlers.append(self._add_row)
        self._format_rpm_info(rpm_files, rpm_infos, row_handlers, query)

        return self._rpm_df

    def get_rpm_info(self, rpmfile):
        self._rpm_df = pd.DataFrame(columns=self._columns)
        rpm_infos = run_cmd('rpm -qpi --nosignature %s' % rpmfile)

        self._format_rpm_info([rpmfile.name], rpm_infos, [self._add_row])

        return self._rpm_df


class DriverReader:
    def __init__(self, logger, progress):
        self._logger = logger
        self._progress = progress
        self._columns = ['Name',
                        'Path',
                        'Flag: supported',
                        'SUSE Release',
                        'Running',
                        'RPM Information']

    def _connect(self, hostname, user, password, ssh_port):
        self._ssh = paramiko.SSHClient()
        self._ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            self._ssh.connect(hostname=hostname,
                             username=user,
                             password=password,
                             port=ssh_port)
            return True
        except paramiko.ssh_exception.SSHException:
            self._logger.error("Can not connect to server: %s", hostname)
            return False

    def _query_filter(self, supported, query='all'):
        if query == 'all':
            return True
        elif query == 'suse' and supported == 'yes':
            return True
        elif query == 'vendor' and supported == 'external':
            return True
        elif query == 'unknow' and (supported != 'yes' and supported != 'no'):
            return True

        return False

    def get_remote_drivers(self, ip='127.0.0.1', user='', password='',
                           ssh_port=22, query='all'):
        if not self._connect(ip, user, password, ssh_port):
            return None

        drivers_modinfo = run_cmd(get_cmd_all_drivers_modinfo(), self._ssh)
        running_drivers_modinfo = run_cmd(get_cmd_all_running_drivers_modinfo(), self._ssh)

        driver_table = self._fill_driver_info(ip, drivers_modinfo,
                                              running_drivers_modinfo,
                                              query, True)

        return driver_table

    def get_local_drivers(self, query='all', row_handlers=[]):
        drivers_modinfo = run_cmd(get_cmd_all_drivers_modinfo())
        running_drivers_modinfo = run_cmd(get_cmd_all_running_drivers_modinfo())

        driver_table = self._fill_driver_info('local host',
                                              drivers_modinfo,
                                              running_drivers_modinfo, query)

        return driver_table

    def _modinfo_to_list(self, raw_output):
        raw_output = str(raw_output, 'utf-8')
        raw_output = list(raw_output.split('filename:'))

        return raw_output[1:]

    def _get_driver_files(self, running_driver_info):
        files = []
        for driver in running_driver_info:
            driver = driver.splitlines()
            files.append(driver[0].lstrip().rstrip())

        return files
    
    def _fill_driver_rpm_info(self, driver_files,
                              item_handler, rpm_table, query, remote):
        start = 0
        step = 1000
        finished = False
        total = len(driver_files)
        while not finished:
            if start+step >= total:
                end = total - 1
                finished = True
            else:
                end = start + step

            cmd = 'rpm -qf ' + ' '.join(driver_files[start: end])
            if remote:
                async_run_cmd(cmd, item_handler,
                              rpm_table, start, query, self._ssh)
            else:
                async_run_cmd(cmd, item_handler, rpm_table, start, query)

            start = end + 1

    def _add_row_handler(self, rpm_table, rpm_info, index, query):
        if rpm_info == '':
            return

        supported = rpm_table[index]['supported']
        self._progress.advance(self._task)
        if self._query_filter(supported, query):
            row = [rpm_table[index]['name'],
                   rpm_table[index]['path'],
                   supported,
                   rpm_table[index]['suserelease'],
                   rpm_table[index]['running'], rpm_info]
            self._driver_df = self._driver_df.append(pd.Series(row, index=self._columns), ignore_index=True)

            self._progress.console.print(f"[light_steel_blue]Found driver: {rpm_table[index]['path']}[/light_steel_blue]")

    def _fill_driver_info(self, ip, drivers_modinfo,
                          running_drivers_modinfo, query='all', remote=False):
        drivers_modinfo = set(self._modinfo_to_list(drivers_modinfo))
        running_drivers_modinfo = set(self._modinfo_to_list(running_drivers_modinfo))

        drivers_modinfo = drivers_modinfo.union(running_drivers_modinfo)
        total_drivers = len(drivers_modinfo)
        self._task = self._progress.add_task("[italic][bold][green] Working on: "
                                             + ip
                                             + "; Total Drivers: "
                                             + str(total_drivers),
                                             total=total_drivers)

        driver_files = self._get_driver_files(drivers_modinfo)
        running_driver_files = self._get_driver_files(running_drivers_modinfo)

        rpm_table = []
        for driver in drivers_modinfo:
            driver = driver.splitlines()
            filename = driver[0].lstrip().rstrip()
            name = ''
            supported = 'Missing'
            suserelease = 'Missing'
            running = str(filename in running_driver_files)

            driver = driver[1:]

            for item in driver:
                values = item.split(':')
                if len(values) < 2:
                    continue

                if values[0] == "supported":
                    supported = ":".join(values[1:]).lstrip()
                elif values[0] == "suserelease":
                    suserelease = ":".join(values[1:]).lstrip()
                elif values[0] == "name":
                    name = ":".join(values[1:]).lstrip()

            if name == '':
                name = Path(filename).name
            rpm_set = {"name": name,
                       "path": filename,
                       "supported": supported,
                       "suserelease": suserelease,
                       "running": running,
                       "rpm_info": ""}

            rpm_table.append(rpm_set)

        self._driver_df = pd.DataFrame(columns=self._columns)
        self._fill_driver_rpm_info(driver_files, self._add_row_handler,
                                   rpm_table, query, remote)

        return self._driver_df



import click
import json
from utils import remote_check
from utils import data_exporter
from utils import data_reader
import os
import logging
import socket
from pathlib import Path
from rich.logging import RichHandler
from rich.live import Live
from utils import terminal_visualizer
from rich.progress import Progress, BarColumn
from rich.style import Style

QUERY_TYPES = ['suse', 'other', 'unknown', 'all']
FORMAT_TYPES = {'html': '.html',
                'excel': '.xlsx',
                'pdf': '.pdf',
                'all': None,
                }


class Check_Target():
    def __init__(self, target):
        if target is None:
            target = 'system'

        self.target = Path(target)

    @property
    def system(self):
        return self.target.name == 'system' and not (self.target.is_file() or self.target.is_dir())

    @property
    def rpm(self):
        # need better check than this
        if self.target.is_file() and self.target.name.endswith('.rpm'):
            return self.target
        return None

    @property
    def dir(self):
        if self.target.is_dir():
            return self.target
        return None

    @property
    def config(self):
        try:
            with self.target.open() as f:
                return json.load(f)
        except Exception as e:
            raise(e)


def with_format_suffix(path, format_type):
    suffix = FORMAT_TYPES.get(format_type, None)

    if suffix is None:
        return path

    if path.suffix == suffix:
        return path

    return path.parent / (path.name + suffix)


def export(exporter, check_result, out_format, dst):
    dst = with_format_suffix(dst, out_format)
    if out_format == 'html':
        exporter.to_html(check_result, dst)
    elif out_format == 'excel':
        exporter.to_excel(check_result, dst)
    elif out_format == 'pdf':
        exporter.to_pdf(check_result, dst)
    elif out_format == 'all':
        exporter.to_all(check_result, dst)


def dst_is_ok(dst, out_format):

    def check(dst):
        if dst.is_file():
            if not os.access(dst, os.W_OK):
                raise Exception('Cannot write to %s' % dst)
            return click.confirm('Overwrite %s?' % dst)
        return True

    if out_format == 'all':
        for format_type, ext in FORMAT_TYPES.items():
            if ext is None:
                continue
            dst = dst.parent / (dst.name + ext)
            if not check(dst):
                return False
        return True

    else:
        ext = FORMAT_TYPES[out_format]
        dst = dst.parent / (dst.name + ext)
        return check(dst)


@click.command()
@click.argument('check_target', default='system')
@click.option('--format', '-f', 'out_format', type=click.Choice(FORMAT_TYPES),
              default='html',
              help='Specify output format')
@click.option('--query', '-q', type=click.Choice(QUERY_TYPES),
              default='all',
              help='Filter results based on vendor tag '
                   'from rpm package providing module. '
                   '"suse" = SUSE, '
                   '"other" = other vendors, '
                   '"unknown" = vendor is unknown, '
                   '"all" = show all (default)'
              )
@click.option('--output', '-o', default='.',
              help='Output destination. Target can be filename or point '
                   'existing directory '
                   'If directory, files will be placed in the directory '
                   'using a autmatically generated filename. If target '
                   'is not an existing directory, file name is assumed '
                   'and output files will use the path and file name '
                   'specified. In either case, the file extension will '
                   'be automatically appended matching on the output format'
              )
def run(check_target, output, out_format, query):
    """Run checks against CHECK_TARGET.

    \b
    CHECK_TARGET can be:
      rpm file
      directory containing rpm files
      "system" to check locally installed kernel modules
      a config file listing remote systems to check

      default is local system
    """

    FORMAT = '%(asctime)-15s %(message)s'
    logging.basicConfig(format=FORMAT, handlers=[RichHandler()])
    logger = logging.getLogger('rich')
    logger.setLevel(logging.INFO)

    target = Check_Target(check_target)

    query = query.lower()

    dst = Path(output)
    if dst.is_dir() or output.endswith('/'):
        dst = dst / 'check_result'

    if not dst_is_ok(dst, out_format):
        logger.error("Can't write to output")
        exit(1)

    # try to guess format from file extension
    if out_format is None:
        ext_to_format = {v: k for k, v in FORMAT_TYPES.items()}
        out_format = ext_to_format.get(dst.suffix, None)

    if target.rpm:
        rpmCheck = data_reader.RPMReader(logger)
        check_result = rpmCheck.get_rpm_info(target.rpm)
        print(check_result)

    elif target.dir:
        rpmCheck = data_reader.RPMReader(logger)
        to_terminal = terminal_visualizer.RPMTerminal()
        with Live(to_terminal.get_table()):
            check_result = rpmCheck.get_rpms_info(
                path=target.dir,
                row_handlers=[to_terminal.add_row],
                query=query)
        exporter = data_exporter.RPMsExporter(logger)
        export(exporter, check_result, out_format, dst)

    elif target.system:
        hostname = socket.gethostname()
        ip = socket.gethostbyname(hostname)
        label = '%s (%s)' % (hostname, ip)
        logger.info('Retrieving kernel module data for %s' % label)
        with Progress() as progress:
            driverCheck = data_reader.DriverReader(logger, progress)
            check_result = {label: driverCheck.get_local_drivers(query)}
        exporter = data_exporter.DriversExporter(logger)
        export(exporter, check_result, out_format, dst)

    elif target.config is not None:
        servers = target.config['servers']
        check_result = remote_check.check_remote_servers(logger, servers)
        exporter = data_exporter.DriversExporter(logger)
        export(exporter, check_result, out_format, dst)


if __name__ == '__main__':
    run()



from rich.live import Live
from rich.table import Table
from rich.progress import Progress


class RPMTerminal:
    def __init__(self):
        self._table = Table(show_header=True,
                            header_style='bold green', show_lines=True)
        self._table.add_column('Name', width=32)
        self._table.add_column('Path', width=64)
        self._table.add_column('Vendor', width=12)
        self._table.add_column('Signature', width=28)
        self._table.add_column('Distribution', width=12)
        self._table.add_column('Driver Flag: supported')
        self._table.add_column('Symbols Check')

    def get_table(self):
        return self._table

    def add_row(self, dataset):
        name = dataset[0]
        path = dataset[1]
        vendor = dataset[2]
        signature = dataset[3]
        distribution = dataset[4]
        supported = dataset[5]
        symbols = dataset[6]

        ds_formatting = ''
        for driver in supported.split('\n'):
            values = driver.split(': ')

            if len(values) < 2:
                ds_formatting += '[red]' + driver + '[/red]' + '\n'
            elif values[1] == 'yes':
                ds_formatting += '[green]' + driver + '[/green]' + '\n'
            elif values[1] == 'external':
                ds_formatting += '[blue]' + driver + '[/blue]' + '\n'
            elif values[1] == 'no' or values[1] == 'Missing':
                ds_formatting += '[red]' + driver + '[/red]' + '\n'

        self._table.add_row(name, path, vendor,
                            signature, distribution, ds_formatting, symbols)



from setuptools import setup, find_packages


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="soliddriver-checks",
    version="0.0.14",
    author="Hui-Zhi Zhao",
    author_email="hui.zhi.zhao@suse.com",
    description=("Check RPM(s) and Drivers information"),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/suse/soliddriver-checks",
    project_urls={
        "Bug Tracker": "https://github.com/suse/soliddriver-checks/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    data_files=[
        ('config', ['src/soliddriver_checks/config/soliddriver-checks.conf'])
    ],
    python_requires=">=3.6",
    install_requires=[
        'Click',
    ],
    entry_points={
        'console_scripts': [
            'soliddriver-checks=soliddriver_checks:run'
        ]
    },
)
